@@ -196,7 +196,8 @@ class Clone(Function, Module):
 
 class SlidingWindowView(Function, Module):
     """
-    Smart Vectorized Sliding Window View with AutoDiff Support using as_strided.
+    Smart Vectorized Sliding Window View with AutoDiff Support and
+    sliding view buffer to avoid unnecessary memory allocation.
     """
     def __init__(self, window_shape: Sequence[int],
                  axes: Union[int, Tuple[int, ...]] = (2, 3),
@@ -209,30 +210,20 @@ class SlidingWindowView(Function, Module):
         self.window_shape = window_shape if isinstance(window_shape, tuple) else \
                            tuple(window_shape for _ in range(len(axes)))
     def forward(self, A: xp.ndarray) -> xp.ndarray:
-        # Compute output shape and strides
-        new_shape = list(A.shape)
-        new_strides = list(A.strides)
-        for i, ax in enumerate(self.axes):
-            new_shape[ax] = (A.shape[ax] - self.window_shape[i]) // self.strides[i] + 1
-            new_strides[ax] = A.strides[ax] * self.strides[i]
-        # Append window dimensions
-        for i, ax in enumerate(self.axes):
-            new_shape.append(self.window_shape[i])
-            new_strides.append(A.strides[ax])
-        # Create strided view
-        self.forward_shape = tuple(new_shape)
-        self.forward_strides = tuple(new_strides)
-        return xp.lib.stride_tricks.as_strided(
-            A, shape=self.forward_shape, strides=self.forward_strides
-        )
+        slices = [slice(None)] * A.ndim
+        for ax, stride in zip(self.axes, self.strides):
+            slices[ax] = slice(None, None, stride)
+        self.slices = tuple(slices)
+        return xp.lib.stride_tricks.sliding_window_view(
+            A, self.window_shape, self.axes)[self.slices]
     def backward(self, grad_output: xp.ndarray) -> xp.ndarray:
         A = self.parent_tensors[0]
         grad_buffer = xp.zeros(A.shape, dtype=grad_output.dtype)
-        # Create strided view for gradient accumulation
-        grad_view = xp.lib.stride_tricks.as_strided(
-            grad_buffer, shape=self.forward_shape, strides=self.forward_strides
-        )
-        # Accumulate gradients
+        kwargs = {"writeable": True} if xp is np else {}
+        grad_view = xp.lib.stride_tricks.sliding_window_view(
+            grad_buffer, self.window_shape, self.axes, **kwargs
+        )[self.slices]
+        # Accumulate gradients using cached view
         grad_view += grad_output
         return grad_buffer
     
